﻿{$reference PresentationCore.dll}
program MuHonix;

uses GraphABC, Sounds;                                      // Подключаем модули графики, звуки.

const
  MaxLives = 3;                                             // Начальное количество жизней паука.
  MaxPercent = 75;                                          // Максимальный процент заполнения поля для завершения текущего уровня.
  LevelDelay = 2000;                                        // Время задержки между уровнями, в миллисекундах.
  QuoteDelay = 5000;                                        // Время задержки для начальной цитаты паука.
  NorthEast = 0; NorthWest = 1; SouthWest = 2; SouthEast = 3;//Мухи двигаются на: северо-восток, северо-запад, юго-запад, юго-восток.
  North = 0; West = 1; South = 2; East = 3;                 // Паук двигается на: север, запад, юг, восток.
  Cell  = 32;                                               // Размер ячейки паутины, желательно кратный 8: 32, 24, 16.
  CellsToLeft = 2;                                          // Сколько клеток отступить снизу?
  //Margin = Cell * CellsToLeft;
//  CellsHeight = (Height div Cell) - CellsToLeft;            // кол-во клеток по высоте.
//  CellsWidth = Width div Cell;                              // кол-во клеток по ширине.
  QuoteColor = clBlue;                                      // Цвет текста цитат.
  GameColor  = clPeru;                                      // Цвет фона во время самой игры.
  MenuColor  = clLightCyan;                                 // цвет фона меню.
  StatsColor = clSaddleBrown;                               // Цвет фона статистики.
  TextColor  = clYellow;                                    // Цвет текста статистики.
  NewWebColor= clBurlyWood;                                 // Цвет новой паутины паука.
  OldWebColor= clWhite;                                     // Цвет старой паутины паука.
  SpiderSize = Cell div 3;
  MinFlyCount = 2;                                          // Минимальное количество мух.
  MaxFlyCount = 10;                                         // Максимальное количество мух.
  FlyDelay = 100;                                           // время в миллисекундах для мух.
  FreeCell    = 0;                                          // Код пустой ячейки.
  OldWebCell  = 1;                                          // Код ячейки со старой паутиной.
  NewWebCell  = 2;                                          // Код ячейки с новой паутиной, рисуемой пауком.
  GasCell     = 3;                                          // Код ячейки с т.н. 'газом'. Нужен для закрашивания паутины.
  ScreenShot  = 'MuHonix.tmp';                              // Временный файл для сохранения экрана при выводе картинок.

var
  LevelNumber: integer;                                     // Номер уровня.
  FlyCount: integer;                                        // Текущее количество мух: растёт с MinFlyCount до MaxFlyCount с каждым уровнем.
  FliesDir: array[1..MaxFlyCount] of integer;               // Здесь хранятся направления движения мух.
  Choice: integer;                                          // То, что выбирает игрок в центральном меню игры.
  SpiderX, SpiderY, SpiderDir: integer;                     // координаты паука на сетке и направление его движения.
  SpiderIsInMotion: boolean;                                // Признак движения паука.
  LevelComplete: boolean;                                   // Признак завершения уровня: при его наступлении начинается новый.
  Width: integer;                                           // Ширина окна программы.
  Height: integer;                                          // Высота окна программы.
  CellsHeight: integer;                                     // кол-во клеток по высоте.
  CellsWidth: integer;                                      // кол-во клеток по ширине.
  Field: array [,] of integer;                              // Игровое поле.
  FliesX: array[1..MaxFlyCount] of integer;                 // X-координаты всех мух.
  FliesY: array[1..MaxFlyCount] of integer;                 // Y-координаты всех мух.
  FliesStuck: array[1..MaxFlyCount] of boolean;             // Признаки остановки мух: true - застряла, false - ещё двигается.
  PressedKey: char;                                         // нажатая клавиша.
  SpiderLives: integer;                                     // текущее количество жизней паука.
  LivesString: String;
  Image: Picture;
  MusicFile: string;

//procedure DrawStatistics;
//const Statistics = Height - CellCount * Cell;
//begin

//end;

procedure KeyDown(Key: char);
begin
  PressedKey := Key;
  case Key of 
    'a', 'A', 'ф', 'Ф': begin spiderDir := West;  SpiderIsInMotion := True end;
    'd', 'D', 'в', 'В': begin spiderDir := East;  SpiderIsInMotion := True end;
    'w', 'W', 'ц', 'Ц': begin spiderDir := North; SpiderIsInMotion := True end;
    's', 'S', 'ы', 'Ы': begin spiderDir := South; SpiderIsInMotion := True end; 
  end;
end;


procedure FloodRegion;
var
  x, y, BurntCellsCount, FlyNumber: integer;
begin
// Для начала, превратим всю новую паутину в старую, а всё пустое пространство - в 'газ'.
  for y := 1 to CellsHeight - 2 do
    for x := 1 to CellsWidth - 2 do
      if      Field[x, y] = FreeCell   then             // Эта ячейка пустая?
              Field[x, y] := GasCell                    // Да, пустая. Значит, превращаем её в 'газ'.
      else if Field[x, y] = NewWebCell then             // Мы нашли новую паутину?
              Field[x, y] := OldWebCell;                // Да, нашли! Превращаем её в старую.

  for FlyNumber := 1 to FlyCount do begin
    Field [FliesX [FlyNumber], FliesY [FlyNumber]] := FreeCell;
    // Здесь мы применяем закон физики, гласящий, что любой газ стремится заполнить собой всё отведённое ему пространство.
    // В данном случае, 'газ' уже заполнил все пустоты без разбора, так что сейчас мы начнём его 'сжигать' при помощи мух.
    // Каждая муха станет пламенем, которое сожжёт весь прилежащий 'газ', оставив его нетронутым лишь в тех пустотах, где мух нет.
    repeat
      BurntCellsCount := 1;                             // Количество ячеек со сгоревшим 'газом'.
      for y := 1 to CellsHeight - 2 do begin            // Начинаем искать в игровом поле пустые ячейки.
        for x := 1 to CellsWidth - 2 do begin
          if Field [x, y] = FreeCell then begin         // Мы нашли пустую ячейку? Отлично!
            if Field [x - 1, y] = GasCell then begin    // Слева 'газ'?
               Field [x - 1, y] := FreeCell;            // Да! Сжигаем, превращаем в пустоту.
               BurntCellsCount += 1;
            end;
            if Field [x + 1, y] = GasCell then begin    // Справа 'газ'?
               Field [x + 1, y] := FreeCell;            // Да! Сжигаем, превращаем в пустоту.
               BurntCellsCount += 1;
            end;
            if Field [x, y - 1] = GasCell then begin    // Сверху 'газ'?
               Field [x, y - 1] := FreeCell;            // Да! Сжигаем, превращаем в пустоту.
               BurntCellsCount += 1;
            end;
            if Field [x, y + 1] = GasCell then begin    // Снизу 'газ'?
               Field [x, y + 1] := FreeCell;            // Да! Сжигаем, превращаем в пустоту.
               BurntCellsCount += 1;
            end;
          end;
          if BurntCellsCount > 1 then break;
        end;
        if BurntCellsCount > 1 then break;
      end;
    until BurntCellsCount = 1;                          // Заполняем пустотой области вокруг каждой из мух до предела.
  end;

  for y := 1 to CellsHeight - 2 do                      // Ищем ячейки с газом и превращаем их в старую паутину.
    for x := 1 to CellsWidth - 2 do
      if Field [x, y] = GasCell then                    // Нашли ячейку с 'газом'? Превращаем её в пустую.
        Field [x, y] := OldWebCell;

// Теперь выводим результат закрашивания на экран.
  for y := 1 to CellsHeight - 2 do
    for x := 1 to CellsWidth - 2 do begin
      case Field[x, y] of
        FreeCell:   SetBrushColor (GameColor);
        OldWebCell: SetBrushColor (OldWebColor);
        NewWebCell: SetBrushColor (NewWebColor);
        GasCell:    SetBrushColor (clBlue);
      end;
      Rectangle(x * Cell, y * Cell, (x + 1) * Cell - 1, (y + 1) * Cell - 1);
    end;

end;

// А эта процедура выводит загадочное сообщение.
procedure DrawFirstScreen;
const
  TotalMessages = 5;
var
  Quote: string;
begin
  Image := Picture.Create ('SpiderQuote.png');
  Image.Draw (0, 0);
  
  SetFontSize (30);
  TextOut (Width div 2 - 50, 20, 'MuHonix');
  SetFontSize (20); SetFontColor (clRed);
  TextOut (Width div 2 - 320, Height - 200, 'Вы же умеете нажимать клавиши W, S, A, D?');
  SetFontColor (clBlack);
  TextOut (Width div 2 - 400, Height - 70, '© 2020-21 Тетюхина Анастасия & Алексей В. Хошенко');
  TextOut (Width div 2 - 300, Height - 30, '© 1986 Эндрю Ллойд Уэббер (музыка)');
  
  Randomize;
  case (Random(TotalMessages)) of
    0: Quote := 'Хочу кушоцъ... запеканку из мух!';
    1: Quote := 'Время обеда - пора похрустеть мухами!';
    2: Quote := 'Мухи - очень уж вкусный предмет!';
    3: Quote := 'Мммухи, хорошо-о-о ли вам меня ви-и-и-дно?';
    4: Quote := 'Хочу муховый омлет!'
  end;
  
  SetFontColor (QuoteColor);
  TextOut ((1200 - TextWidth (Quote)) div 2, 300, Quote);
  Sleep (QuoteDelay);
  
  //SetBrushColor (Background);
  //SetPenColor (Background);
  //  TextOut (Width div 3, Height div 2, Quote);
end;

function DrawMenu: integer;
begin
  SetBrushColor(clDarkCyan);
  //  SetPenColor (clWhite);
  TextOut(Width - 250, 10, 'Начать игру      ');
  SetBrushColor(GameColor);
  TextOut(Width - 250, 60, 'Закончить игру');
  TextOut(Width - 250, 110, 'Об авторах');
  Sleep(3000);
  DrawMenu := 1;        // Пока что мы выбираем 1-й пункт, т.е. "Начать игру".
end;

procedure DrawField;
var
  x, y, i: integer;
begin
  for y := 1 to CellsHeight - 2 do 
    for x := 1 to CellsWidth - 2 do 
      Field[x, y] := FreeCell;                                          // внутренняя часть поля.
  
  for y := 0 to CellsHeight - 1 do
  begin
    Field[0,              y] := OldWebCell;                             // левая сторона поля.
    Field[CellsWidth - 1, y] := OldWebCell;                             // правая сторона поля.
    //Field[CellsWidth div 2 - 1, y]:= OldWebCell;                      // отладка: средняя линия.
  end;
  
  for x := 0 to CellsWidth - 1 do
  begin
    Field[x,               0] := OldWebCell;                            // верхняя сторона поля.
    Field[x, CellsHeight - 1] := OldWebCell;                            // нижняя сторона поля.
  end;
  
  
  // Таперича рисуем поле, глядя на его массив Field.
  // SetPenWidth (1);
  SetPenColor(GameColor);
  for y := 0 to CellsHeight - 1 do
    for x := 0 to CellsWidth - 1 do
    begin
      if Field[x, y] = OldWebCell then                                 // Что здесь?
        SetBrushColor(OldWebColor)                                     // Паутина!
      else
        SetBrushColor(GameColor);                                      // Пустота.
      Rectangle(x * Cell, y * Cell, (x + 1) * Cell - 1, (y + 1) * Cell - 1);
    end;
  
  //ставим паука в 1 из 4 углов
  SpiderIsInMotion := false;                                           // Паук пока не двигается и ждёт нажатия клавиши.
  Randomize;
  case Random(4) of
    0: begin SpiderX := Width div Cell - 1; SpiderY := 0; end;
    1: begin SpiderX := Width div Cell - 1; SpiderY := CellsHeight - 1; end;
    2: begin SpiderX := 0; SpiderY := CellsHeight - 1; end;
    3: begin SpiderX := 0; SpiderY := 0; end;
  end;
  SetBrushColor(clDarkBlue);
  FillCircle(SpiderX * Cell + Cell div 2, SpiderY * Cell + Cell div 2, SpiderSize);
  
  //а теперь расставляем мух
  SetBrushColor(clPurple);
  for i := 1 to FlyCount do
  begin
    FliesX[i] := 2 + random(CellsWidth - 2 - 1);            //-1 - корректирующая единица,
    FliesY[i] := 2 + random(CellsHeight - 3 - 1);           // потому что массивы нумеруются с 1.
    FliesDir[i] := random(4);                               //выбираем направление для мухи.
    FliesStuck [i] := false;                                // true - муха застряла, false - ещё двигается.
    FillEllipse(FliesX[i] * Cell + Cell div 5,                     //x1 рисуемой мухи.
                FliesY[i] * Cell + Cell div 3,                     //y1
                FliesX[i] * Cell + Cell div 4 * 3 + Cell div 5,    //x2
                FliesY[i] * Cell + Cell div 4 * 3);                //y2
  end;
end;//завершаем DrawField.

procedure MainCycle;
var
  FlyTime, FlyNumber,
  x, y,
  Fillment, CurrPercent,
  deltaX, deltaY,               // сдвиг координат X и Y, в зависимости от направления движения.
  newX, newY,                   // координаты, куда желает попасть паук.
  LastWebX, LastWebY: integer;  // последние координаты в белой паутине, где был замечен паук - нужны для возврата в случае форс-мажора.
  MessageString: string;
  FlyRuinsWeb: boolean;         // Признак того, что одна из мух испортила пауку его новую паутину.
begin
  //For FlyTime := 1 to 100 do                                        // Отладка: пока двигаемся 100 раз.
  //begin
  
  LevelComplete := false;                                             // Раз уровень только начинается, то он ещё не завершён.
  FlyTime := 1;
  
  // Начало главного цикла игры.
  repeat
    FlyRuinsWeb := false;                                             // Пока что ни одна муха не испортила пауку праздник.
    
    SetBrushColor(StatsColor);                                        // фон для статистики.
    SetFontColor(TextColor);                                          // цвет текста.
    MessageString := 'Время игры: ' + FlyDelay * FlyTime div 1000 + ' ';
    TextOut(Width div 4 * 0, Height - (2 * cell), MessageString);

// Мухи, кругом одни мухи!
    for FlyNumber := 1 to FlyCount do                        //обрабатываем мух по 1.
    begin
      case FliesDir[FlyNumber] of 
        NorthEast: begin deltaY := -1; deltaX := +1; end; 
        NorthWest: begin deltaY := -1; deltaX := -1; end;
        SouthWest: begin deltaY := +1; deltaX := -1; end;
        SouthEast: begin deltaY := +1; deltaX := +1; end;
      end;
      
      //теперь проверяем, можно ли мухе двигаться в заданном направлении.
      newX := FliesX [FlyNumber] + deltaX;                  //желаемый X для текущей мухи.
      newY := FliesY [FlyNumber] + deltaY;                  //желаемый Y для текущей мухи.
      
      if newX < 0 then newX := 0;
      if newY < 0 then newY := 0;
      if newX > CellsWidth  then newX := CellsWidth;    // убедиться в правильности проверки
      if newY > CellsHeight then newY := CellsHeight;   // убедиться в правильности проверки
      
      SetBrushColor(GameColor);               //стираем старую муху.
      SetPenColor(GameColor);
      rectangle(FliesX[FlyNumber] * Cell + 1,        FliesY[FlyNumber] * Cell + 1,
                FliesX[FlyNumber] * Cell + Cell - 1, FliesY[FlyNumber] * Cell + Cell - 2);
      
      if (Field[newX, newY] = NewWebCell) or         // Тут новая паутина?
         ((SpiderX = FliesX[FlyNumber]) and          // Или же муха угодила...
          (SpiderY = FliesY[FlyNumber])) then begin  // ... прямо в паука?
        FlyRuinsWeb := true;                         // Значит, муха испортила пауку праздник!
        
        if ((SpiderX = FliesX[FlyNumber]) and (SpiderY = FliesY[FlyNumber])) then // Муха врезалась в паука?
            Image := Picture.Create ('FlySlapsSpiderOnFace.png')
        else
          case Random (3) of                                                      // Нет, муха врезалась в паутину.
            0: Image := Picture.Create ('WhyAreYouDoingSo.png');
            1: Image := Picture.Create ('SpiderBite.png');
            2: Image := Picture.Create ('SpiderManiac.png');
          end;

        SaveWindow (ScreenShot);
        Image.Draw ((Width - Image.Width) div 2, (CellsHeight * Cell - Image.Height) div 2);
        System.Console.Beep (250, 200);                // Звук того, что паук вляпался в неприятности.
        System.Console.Beep (300, 800);                // Звук того, что паук вляпался в неприятности.
        Sleep (500);
        LoadWindow (ScreenShot);
      end;
      
      if Field[newX, newY] <> OldWebCell then         // Тут свободно?
        begin
          FliesX[FlyNumber] := newX;
          FliesY[FlyNumber] := newY; 
        end
      else                                            // Тут занято, теперь будем думать: "куда нам дальше?"
        begin
          if Field[FliesX[FlyNumber], FliesY[FlyNumber] + deltaY] = FreeCell then        // столкновение с вертикальной стеной?
            case FliesDir[FlyNumber] of
              NorthEast: FliesDir[FlyNumber] := NorthWest; 
              NorthWest: FliesDir[FlyNumber] := NorthEast; 
              SouthWest: FliesDir[FlyNumber] := SouthEast; 
              SouthEast: FliesDir[FlyNumber] := SouthWest; 
            end
          else                   // столкновение с горизонтальной стеной.
            case FliesDir[FlyNumber] of
              NorthEast: FliesDir[FlyNumber] := SouthEast; 
              NorthWest: FliesDir[FlyNumber] := SouthWest; 
              SouthWest: FliesDir[FlyNumber] := NorthWest; 
              SouthEast: FliesDir[FlyNumber] := NorthEast; 
            end;
          
          if ((Field [FliesX [FlyNumber] + deltaX, FliesY [FlyNumber] + deltaY] = OldWebCell) and
              (Field [FliesX [FlyNumber] - deltaX, FliesY [FlyNumber] - deltaY] = OldWebCell)) and
             ((Field [FliesX [FlyNumber] - deltaX, FliesY [FlyNumber] + deltaY] = OldWebCell) and
              (Field [FliesX [FlyNumber] + deltaX, FliesY [FlyNumber] - deltaY] = OldWebCell))
             and (FliesStuck [FlyNumber] = false) then begin
            FliesStuck [FlyNumber] := true;                                     // Похоже, эта муха застряла.
            Image := Picture.Create ('FlyIsCaught.png');
            SaveWindow (ScreenShot);
            Image.Draw ((Width - Image.Width) div 2, (CellsHeight * Cell - Image.Height) div 2);
            System.Console.Beep (1000, 200);                                    // Звук того, что муха застряла.
            System.Console.Beep (1300, 800);                                    // Звук того, что паук застряла.
            Sleep (500);
            LoadWindow (ScreenShot);
          end;
        end;
      
      SetBrushColor(clPurple);
      FillEllipse(    FliesX[FlyNumber] * Cell + Cell div 5,                    //x1 рисуемой мухи.
                    FliesY[FlyNumber] * Cell + Cell div 3,                      //y1
                      (FliesX[FlyNumber] + 1) * Cell - Cell div 5,              //x2
                    FliesY[FlyNumber] * Cell + Cell div 4 * 3);                 //y2
    end;

// здесь логика движения паука.
    if SpiderIsInMotion = True then begin                             //паук желает двигаться?
      case SpiderDir of 
        North: begin deltaX := 0; deltaY := -1; end;
        South: begin deltaX := 0; deltaY := +1; end;
        West:  begin deltaX := -1; deltaY := 0; end;
        East:  begin deltaX := +1; deltaY := 0; end;
      end;
      newX := SpiderX + deltaX;
      newY := SpiderY + deltaY;
      if newX < 0 then newX := 0;
      if newY < 0 then newY := 0;
      if newX >= CellsWidth then newX := CellsWidth - 1;     // убедиться в правильности проверки.
      if newY >= CellsHeight then newY := CellsHeight - 1;   // убедиться в правильности проверки.

      SetPenColor(GameColor);
      
      if (Field[SpiderX, SpiderY] = FreeCell) and (not FlyRuinsWeb) then // Здесь пустая ячейка и ни одна муха пауку пока не мешает?
        begin
          Field[SpiderX, SpiderY] := NewWebCell;                // Да! Значит, рисуем новую паутину.
          SetBrushColor(NewWebColor);
          Rectangle(SpiderX * Cell, SpiderY * Cell, (SpiderX + 1) * Cell - 1, (SpiderY + 1) * Cell - 1);
          
          if (Field [SpiderX, SpiderY] = NewWebCell) and
             (Field [NewX,    NewY   ] = OldWebCell) then begin // Паук вернулся из командировки?
              FloodRegion;                                      // Да! Значит, пора закрашивать только что отрезанную им область.
          end;
          
          SpiderX := newX;
          SpiderY := newY;
        end
      else                                                      // Нет! Значит, стираем паука и возвращаем его домой.
        begin                                            
          if (Field[SpiderX, SpiderY] = NewWebCell) or (FlyRuinsWeb = true) then
            begin
              SpiderLives := SpiderLives - 1;                   // Кол-во жизней мы уменьшили, надо бы это показать.
              if SpiderLives < 0 then SpiderLives := 0;         // Подушка безопасности: количество жизней паука не может быть < 0.
              LivesString := 'Жизней: ' + SpiderLives + ' ';
              SetBrushColor(StatsColor);                        // фон для статистики.
              SetFontColor(TextColor);                          // по плану - цвет текста.
              TextOut(Width div 4 * 1 + TextWidth (LivesString) div 2, Height - (2 * cell), LivesString);
             
              if SpiderLives <= 0 then                          // Жизни у паука закончились?
                 PressedKey := chr(27);                         // Имитируем нажатие клавиши Esc.
              
              SpiderIsInMotion := false;                        // Тпру! Приехали - останавливаем паука.
              newX := LastWebX;                                 // Ставим паука на то место, откуда он
              newY := LastWebY;                                 // начал свою последнюю новую паутину.
              
              if Field[SpiderX, SpiderY] = NewWebCell then begin
                SaveWindow (ScreenShot);
                Image := Picture.Create ('SpiderSuicide.png');
                Image.Draw ((Width - Image.Width) div 2, (CellsHeight * Cell - Image.Height) div 2);
                System.Console.Beep (300, 200);                // Звук того, что паук вляпался в неприятности.
                System.Console.Beep (250, 800);                // Звук того, что паук вляпался в неприятности.
                Sleep (500);
                LoadWindow (ScreenShot);
              end;
              
              for y := 1 to CellsHeight - 1 do
                for x := 1 to CellsWidth - 1 do
                  if Field[x, y] = NewWebCell then begin        // нашли новую паутину?
                    Field[x, y] := FreeCell;                    // тогда стираем.
                    SetBrushColor (GameColor);                  // а теперь мы эту ячейку зарисовываем.
                    Rectangle(X * Cell, Y * Cell, (X + 1) * Cell - 1, (Y + 1) * Cell - 1);
                  end;
              SetBrushColor (GameColor);
          end
        else
          begin                                                 // Похоже, здесь белая паутина.
            LastWebX := SpiderX;                                // Запоминаем прежний X паука на случай форс-мажора.
            LastWebY := SpiderY;                                // Запоминаем прежний Y паука на случай форс-мажора.
            SetBrushColor (OldWebColor);                        // Закрашиваем ячейку с пауком.
          end;
        Rectangle(SpiderX * Cell, SpiderY * Cell, (SpiderX + 1) * Cell - 1, (SpiderY + 1) * Cell - 1);
        SpiderX := newX;
        SpiderY := newY;
      end;
      
      SetBrushColor(clDarkBlue);  // Паучок у нас пока тёмно-синий.
      FillCircle(SpiderX * Cell + Cell div 2, SpiderY * Cell + Cell div 2, SpiderSize); // Рисуем паучка.
    end;

// Статистика и прочая логика.    
    Fillment := 0;
    for y := 1 to CellsHeight - 2 do
      for x := 1 to CellsWidth - 2 do 
        if Field [x, y] = OldWebCell then Fillment += 1;
    SetBrushColor(StatsColor);                                             // фон для статистики.
    SetFontColor(TextColor);                                               // цвет текста.
    CurrPercent := (Fillment * 100) div ((CellsHeight - 2) * (CellsWidth - 2)); // Текущий процент заполнения поля.
    MessageString := 'Заполнено: ' + CurrPercent + '% ';
    TextOut(Width div 4 * 2 + TextWidth ('Заполнено: 100%') div 2, Height - (2 * cell), MessageString);
    if CurrPercent >= MaxPercent then LevelComplete := true;
    
    Sleep(FlyDelay);
    FlyTime += 1;
  until (PressedKey = chr(27)) or (LevelComplete = true);
  
  Setbrushcolor(StatsColor);                                               // фон для статистики.
  if LevelComplete = true then                                             // Уровень завершён?
    begin
      SetFontColor(clCyan);                                                   // цвет текста.
      MessageString := ' Уровень завершён! '
    end
  else
    begin
      SetFontColor(clRed);                                                   // цвет текста.
      MessageString := ' Игра закончена! ';
    end;
  TextOut(Width div 2 - 100, Height div 2 - Cell, MessageString);
  Sleep (LevelDelay);
end;

// Здесь начинается основная программа.
begin
//  SetWindowSize(Width, Height);                    //Выбираем размер окна.
//  CenterWindow;                                    // ставим окно посередине.
  MaximizeWindow;
  Width := WindowWidth;                                      // Ширина окна программы.
  Height := WindowHeight;                                    // Высота окна программы.
  CellsHeight := (Height div Cell) - CellsToLeft;            // кол-во клеток по высоте.
  CellsWidth := Width div Cell;                              // кол-во клеток по ширине.
  SetLength (Field, CellsWidth, CellsHeight);

  SetBrushColor(MenuColor);    
  Rectangle(0, 0, Width - 1, Height - 1);

  MusicFile := ExpandFileName ('Music1.mp3');
  PlaySound (MusicFile);
//  var Mus := new System.Windows.Media.MediaPlayer;
//  Mus.Open (new System.Uri ('Music1.mp3'));
//  Mus.Play;

  SetFontSize(20);
  SetFontStyle(fsBold);
  DrawFirstScreen;                               // Выводим случайную цитату паука.
  Rectangle(0, 0, Width - 1, Height - 1);         // Стираем экран.
  
    //Choice := DrawMenu;                            // Выбираем один из пунктов меню.
  //TextOut (Width div 2, Height div 2, Choice);     // Отладка! Проверяем выбор игрока в меню.

    
  OnKeyPress := KeyDown;                                        // Наш обработчик клавиатуры.
  FlyCount := MinFlyCount;                                      // Начинаем с минимального количества мух.
  LevelNumber := 1;                                             // Начальный номер уровня.

  repeat
    SetBrushColor (StatsColor);
    Rectangle (0, 0, Width - 1, Height - 1);                    // Стираем экран.
    DrawField;
    
    SetBrushColor(StatsColor);                                  // фон для статистики.
    SetFontColor(TextColor);                                    // по плану - цвет текста.
    LivesString := 'Жизней: ' + MaxLives + ' ';
    TextOut(Width div 4 * 1 + TextWidth (LivesString) div 2, Height - (2 * cell), LivesString);
    LivesString := 'Уровень: ' + LevelNumber + ' ';
    TextOut(Width div 4 * 4 - TextWidth (LivesString), Height - (2 * cell), LivesString);
    
    SpiderLives := MaxLives;                                     // даём пауку жизни.
//  DrawStatistics;
    PressedKey := chr (0);                                       // Очищаем нажатую клавишу.
    MainCycle;                                                   // запускаем мух.
    if LevelComplete = true then begin
      FlyCount += 1;                                             // В случае удачного завершения уровня увеличиваем количество мух.
      LevelNumber += 1;
    end;
  until (PressedKey = chr(27)) or (FlyCount > MaxFlyCount);
  
  if LevelComplete = true then begin                             // Игра завершена удачно?
//  TextOut(Width div 2 - 200, Height div 2 - Cell, 'Наши поздравления, чемпион!');
    SetBrushColor (MenuColor);
    Rectangle (0, 0, Width - 1, Height - 1);
    Image := Picture.Create ('EndGame.png');
//  Image.SetSize (800, 400);
    Image.Draw ((Width - Image.Width) div 2, (CellsHeight * Cell - Image.Height) div 2);
    SetFontColor (clBlack);
    TextOut (200, 100, 'Я наелся!');
  end;

  DeleteFile (ScreenShot);
  Sleep (LevelDelay * 2);
  Exit;
end.